@@ -104,19 +104,21 @@
     rust_2018_idioms,
     rust_2018_compatibility
 )]
-use kuchiki::{parse_html, traits::TendrilSink, Node, NodeRef, Specificity};
+use kuchiki::{
+    parse_html, traits::TendrilSink, ElementData, Node, NodeDataRef, NodeRef, Specificity,
+};
 
 pub mod error;
 mod parser;
 
+use ahash::AHashMap;
 pub use error::InlineError;
 use smallvec::{smallvec, SmallVec};
 use std::{
     borrow::Cow,
-    collections::{hash_map::Entry, HashMap},
+    collections::hash_map::Entry,
     fs::File,
     io::{Read, Write},
-    ops::Deref,
 };
 pub use url::{ParseError, Url};
 
@@ -140,6 +142,7 @@ pub struct InlineOptions<'a> {
 
 impl<'a> InlineOptions<'a> {
     /// Options for "compact" HTML output.
+    #[must_use]
     #[inline]
     pub const fn compact() -> Self {
         InlineOptions {
@@ -152,36 +155,42 @@ impl<'a> InlineOptions<'a> {
     }
 
     /// Override whether "style" tags should be inlined.
+    #[must_use]
     pub fn inline_style_tags(mut self, inline_style_tags: bool) -> Self {
         self.inline_style_tags = inline_style_tags;
         self
     }
 
     /// Override whether "style" tags should be removed after processing.
+    #[must_use]
     pub fn remove_style_tags(mut self, remove_style_tags: bool) -> Self {
         self.remove_style_tags = remove_style_tags;
         self
     }
 
     /// Set base URL that will be used for loading external stylesheets via relative URLs.
+    #[must_use]
     pub fn base_url(mut self, base_url: Option<Url>) -> Self {
         self.base_url = base_url;
         self
     }
 
     /// Override whether remote stylesheets should be loaded.
+    #[must_use]
     pub fn load_remote_stylesheets(mut self, load_remote_stylesheets: bool) -> Self {
         self.load_remote_stylesheets = load_remote_stylesheets;
         self
     }
 
     /// Set additional CSS to inline.
+    #[must_use]
     pub fn extra_css(mut self, extra_css: Option<Cow<'a, str>>) -> Self {
         self.extra_css = extra_css;
         self
     }
 
     /// Create a new `CSSInliner` instance from this options.
+    #[must_use]
     pub const fn build(self) -> CSSInliner<'a> {
         CSSInliner::new(self)
     }
@@ -210,6 +219,7 @@ pub struct CSSInliner<'a> {
 
 impl<'a> CSSInliner<'a> {
     /// Create a new `CSSInliner` instance with given options.
+    #[must_use]
     #[inline]
     pub const fn new(options: InlineOptions<'a>) -> Self {
         CSSInliner { options }
@@ -234,6 +244,7 @@ impl<'a> CSSInliner<'a> {
     /// # }
     /// # run().unwrap();
     /// ```
+    #[must_use]
     #[inline]
     pub fn options() -> InlineOptions<'a> {
         InlineOptions::default()
@@ -241,8 +252,9 @@ impl<'a> CSSInliner<'a> {
 
     /// Inliner, that will produce "compact" HTML.
     /// For example, "style" tags will be removed.
+    #[must_use]
     #[inline]
-    pub fn compact() -> Self {
+    pub const fn compact() -> Self {
         CSSInliner {
             options: InlineOptions::compact(),
         }
@@ -277,7 +289,8 @@ impl<'a> CSSInliner<'a> {
         // and then reused, which allows O(1) access to find them.
         // Internally, their raw pointers are used to implement `Eq`, which seems like the only
         // reasonable approach to compare them (performance-wise).
-        let mut styles = HashMap::with_capacity(128);
+        let mut styles = AHashMap::with_capacity(128);
+        let mut style_tags: SmallVec<[NodeDataRef<ElementData>; 4]> = smallvec![];
         if self.options.inline_style_tags {
             for style_tag in document
                 .select("style")
@@ -285,19 +298,24 @@ impl<'a> CSSInliner<'a> {
             {
                 if let Some(first_child) = style_tag.as_node().first_child() {
                     if let Some(css_cell) = first_child.as_text() {
-                        process_css(&document, css_cell.borrow().as_str(), &mut styles)?;
+                        process_css(&document, css_cell.borrow().as_str(), &mut styles);
                     }
                 }
                 if self.options.remove_style_tags {
-                    style_tag.as_node().detach()
+                    style_tags.push(style_tag);
                 }
             }
-        } else if self.options.remove_style_tags {
-            for style_tag in document
-                .select("style")
-                .map_err(|_| error::InlineError::ParseError(Cow::from("Unknown error")))?
-            {
-                style_tag.as_node().detach()
+        }
+        if self.options.remove_style_tags {
+            if !self.options.inline_style_tags {
+                style_tags.extend(
+                    document
+                        .select("style")
+                        .map_err(|_| error::InlineError::ParseError(Cow::from("Unknown error")))?,
+                );
+            }
+            for style_tag in &style_tags {
+                style_tag.as_node().detach();
             }
         }
         if self.options.load_remote_stylesheets {
@@ -312,12 +330,12 @@ impl<'a> CSSInliner<'a> {
                 if !href.is_empty() {
                     let url = self.get_full_url(href);
                     let css = load_external(url.as_ref())?;
-                    process_css(&document, css.as_str(), &mut styles)?;
+                    process_css(&document, css.as_str(), &mut styles);
                 }
             }
         }
         if let Some(extra_css) = &self.options.extra_css {
-            process_css(&document, extra_css, &mut styles)?;
+            process_css(&document, extra_css, &mut styles);
         }
         for (node_id, styles) in styles {
             // SAFETY: All nodes are alive as long as `document` is in scope.
@@ -333,7 +351,7 @@ impl<'a> CSSInliner<'a> {
                 .try_borrow_mut()
             {
                 if let Some(existing_style) = attributes.get_mut("style") {
-                    *existing_style = merge_styles(existing_style, &styles)?
+                    *existing_style = merge_styles(existing_style, &styles)?;
                 } else {
                     let mut final_styles = String::with_capacity(128);
                     for (name, (_, value)) in styles {
@@ -359,11 +377,10 @@ impl<'a> CSSInliner<'a> {
             // Use the same scheme as the base URL
             if href.starts_with("//") {
                 return Cow::Owned(format!("{}:{}", base_url.scheme(), href));
-            } else {
-                // Not a URL, then it is a relative URL
-                if let Ok(new_url) = base_url.join(href) {
-                    return Cow::Owned(new_url.into());
-                }
+            }
+            // Not a URL, then it is a relative URL
+            if let Ok(new_url) = base_url.join(href) {
+                return Cow::Owned(new_url.into());
             }
         };
         // If it is not a valid URL and there is no base URL specified, we assume a local path
@@ -389,8 +406,8 @@ type NodeId = *const Node;
 fn process_css(
     document: &NodeRef,
     css: &str,
-    styles: &mut HashMap<NodeId, HashMap<String, (Specificity, String)>>,
-) -> Result<()> {
+    styles: &mut AHashMap<NodeId, AHashMap<String, (Specificity, String)>>,
+) {
     let mut parse_input = cssparser::ParserInput::new(css);
     let mut parser = cssparser::Parser::new(&mut parse_input);
     let rule_list =
@@ -404,17 +421,17 @@ fn process_css(
                 let specificity = matching_elements.selectors.0[0].specificity();
                 for matching_element in matching_elements {
                     let element_styles = styles
-                        .entry(matching_element.as_node().deref())
-                        .or_insert_with(|| HashMap::with_capacity(16));
+                        .entry(&**matching_element.as_node())
+                        .or_insert_with(|| AHashMap::with_capacity(8));
                     for (name, value) in &declarations {
                         match element_styles.entry(name.to_string()) {
                             Entry::Occupied(mut entry) => {
                                 if entry.get().0 <= specificity {
-                                    entry.insert((specificity, value.to_string()));
+                                    entry.insert((specificity, (*value).to_string()));
                                 }
                             }
                             Entry::Vacant(entry) => {
-                                entry.insert((specificity, value.to_string()));
+                                entry.insert((specificity, (*value).to_string()));
                             }
                         }
                     }
@@ -425,7 +442,6 @@ fn process_css(
             // Which means that they will fall into this category and will be ignored
         }
     }
-    Ok(())
 }
 
 impl Default for CSSInliner<'_> {
@@ -449,29 +465,28 @@ pub fn inline_to<W: Write>(html: &str, target: &mut W) -> Result<()> {
 
 fn merge_styles(
     existing_style: &str,
-    new_styles: &HashMap<String, (Specificity, String)>,
+    new_styles: &AHashMap<String, (Specificity, String)>,
 ) -> Result<String> {
-    // Parse existing declarations in "style" attribute
+    // Parse existing declarations in the "style" attribute
     let mut input = cssparser::ParserInput::new(existing_style);
     let mut parser = cssparser::Parser::new(&mut input);
     let declarations =
         cssparser::DeclarationListParser::new(&mut parser, parser::CSSDeclarationListParser);
-    // New rules override old ones and we store selectors inline to check the old rules later
-    let mut buffer: SmallVec<[&str; 8]> = smallvec![];
+    // New rules should not override old ones and we store selectors inline to check the old rules later
+    let mut buffer: SmallVec<[String; 8]> = smallvec![];
     let mut final_styles = String::with_capacity(256);
-    for (property, (_, value)) in new_styles {
-        final_styles.push_str(property);
+    for declaration in declarations {
+        let (name, value) = declaration?;
+        final_styles.push_str(&name);
         final_styles.push(':');
         final_styles.push_str(value);
         final_styles.push(';');
-        // This property won't be taken from existing styles. i.e. it is overridden by new styles
-        buffer.push(property);
+        // This property won't be taken from new styles
+        buffer.push(name.to_string())
     }
-    for declaration in declarations {
-        let (name, value) = declaration?;
-        // Usually this buffer is small and it is faster than checking a {Hash,BTree}Map
-        if !buffer.contains(&name.as_ref()) {
-            final_styles.push_str(&name);
+    for (property, (_, value)) in new_styles {
+        if !buffer.contains(property) {
+            final_styles.push_str(property);
             final_styles.push(':');
             final_styles.push_str(value);
             final_styles.push(';');
