@@ -83,6 +83,32 @@ fn overloaded_styles() {
     )
 }
 
+#[test]
+fn existing_styles() {
+    // When there is a `style` attribute on a tag that contains a rule
+    // And the `style` tag contains the same rule applicable to that tag
+    assert_inlined!(
+        style = "h1 { color: red; }",
+        body = r#"<h1 style="color: blue">Hello world!</h1>"#,
+        // Then the existing rule should be preferred
+        expected = r#"<h1 style="color: blue;">Hello world!</h1>"#
+    )
+}
+
+#[test]
+fn existing_styles_with_merge() {
+    // When there is a `style` attribute on a tag that contains a rule
+    // And the `style` tag contains the same rule applicable to that tag
+    // And there is a new rule in the `style` tag
+    assert_inlined!(
+        style = "h1 { color: red; font-size:14px; }",
+        body = r#"<h1 style="color: blue">Hello world!</h1>"#,
+        // Then the existing rule should be preferred
+        // And the new style should be merged
+        expected = r#"<h1 style="color: blue;font-size:14px;">Hello world!</h1>"#
+    )
+}
+
 #[test]
 fn empty_style() {
     // When the style tag is empty
@@ -127,6 +153,93 @@ fn remove_style_tag() {
     assert_eq!(result, "<html><head><title>Test</title></head><body><h1 style=\"background-color: blue;\">Hello world!</h1></body></html>")
 }
 
+#[test]
+fn remove_multiple_style_tags() {
+    let html = r#"
+<html>
+<head>
+<style>
+h1 {
+    text-decoration: none;
+}
+</style>
+<style>
+.test-class {
+        color: #ffffff;
+}
+a {
+        color: #17bebb;
+}
+</style>
+</head>
+<body>
+<a class="test-class" href="https://example.com">Test</a>
+<h1>Test</h1>
+</body>
+</html>
+    "#;
+    let inliner = CSSInliner::compact();
+    let result = inliner.inline(html).unwrap();
+    assert_eq!(
+        result,
+        r#"<html><head>
+
+
+</head>
+<body>
+<a class="test-class" href="https://example.com" style="color: #ffffff;">Test</a>
+<h1 style="text-decoration: none;">Test</h1>
+
+
+    </body></html>"#
+    )
+}
+
+#[test]
+fn remove_multiple_style_tags_without_inlining() {
+    let html = r#"
+<html>
+<head>
+<style>
+h1 {
+    text-decoration: none;
+}
+</style>
+<style>
+.test-class {
+        color: #ffffff;
+}
+a {
+        color: #17bebb;
+}
+</style>
+</head>
+<body>
+<a class="test-class" href="https://example.com">Test</a>
+<h1>Test</h1>
+</body>
+</html>
+    "#;
+    let inliner = CSSInliner::options()
+        .remove_style_tags(true)
+        .inline_style_tags(false)
+        .build();
+    let result = inliner.inline(html).unwrap();
+    assert_eq!(
+        result,
+        r#"<html><head>
+
+
+</head>
+<body>
+<a class="test-class" href="https://example.com">Test</a>
+<h1>Test</h1>
+
+
+    </body></html>"#
+    )
+}
+
 #[test]
 fn do_not_process_style_tag() {
     let html = html!("h1 {background-color: blue;}", "<h1>Hello world!</h1>");
@@ -190,7 +303,7 @@ h2 { color: red; }
 <h2>Smaller Text</h2>
 </body>
 </html>"#;
-    let result = inline(&html).unwrap();
+    let result = inline(html).unwrap();
     assert!(result.ends_with(
         r#"<body>
 <h1 style="color: blue;">Big Text</h1>
@@ -216,7 +329,7 @@ h2 { color: red; }
 <h2>Smaller Text</h2>
 </body>
 </html>"#;
-    let result = inline(&html).unwrap();
+    let result = inline(html).unwrap();
     assert!(result.ends_with(
         r#"<body>
 <h1 style="color: blue;">Big Text</h1>
@@ -242,7 +355,7 @@ h2 { color: red; }
 <h2>Smaller Text</h2>
 </body>
 </html>"#;
-    let result = inline(&html).unwrap();
+    let result = inline(html).unwrap();
     assert!(result.ends_with(
         r#"<body>
 <h1 style="color: blue;">Big Text</h1>
@@ -273,7 +386,7 @@ h2 { color: red; }
         ..Default::default()
     };
     let inliner = CSSInliner::new(options);
-    let result = inliner.inline(&html).unwrap();
+    let result = inliner.inline(html).unwrap();
     assert!(result.ends_with(
         r#"<body>
 <h1 style="color: blue;">Big Text</h1>
@@ -304,7 +417,7 @@ h2 { color: red; }
         ..Default::default()
     };
     let inliner = CSSInliner::new(options);
-    let result = inliner.inline(&html).unwrap();
+    let result = inliner.inline(html).unwrap();
     assert!(result.ends_with(
         r#"<body>
 <h1 style="color: blue;">Big Text</h1>
@@ -320,8 +433,8 @@ fn customize_inliner() {
         load_remote_stylesheets: false,
         ..Default::default()
     };
-    assert_eq!(options.load_remote_stylesheets, false);
-    assert_eq!(options.remove_style_tags, false);
+    assert!(!options.load_remote_stylesheets);
+    assert!(!options.remove_style_tags);
     assert_eq!(options.base_url, None);
 }
 
